--------------------------------------------------------------------------
-- MSS copyright 1999-2008
-- Filename: COM8000.VHD 
--	Revision: 1
-- Last modified: 6-23-08
-- Inheritance: 	COM-8003 rev12 12-25-05
-- Authors: 
--		Bengt-Arne Bengtsson / MSS
-- 	Alain Zarembowitch / MSS
--		Adam Kwiatkowski / MSS
--		
--	Description: top level for COM-8000 
-- Code template 
-- Specifications are at www.comblock.com/download/COM8000.pdf
--
--------------------------------------------------------------------------
library IEEE;
use IEEE.std_logic_1164.all;  
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;
Library UNISIM;
use UNISIM.vcomponents.all;


entity COM8000 is port (
	--// CLOCK
   CLK_IN1 : in std_logic;	-- master clock for this FPGA, internal to the module

	--// Left connector, 40 pin female 2mm connector. 
	-- Pin 1 is reserved for input clock (90 MHz, DDR clock)
	-- Pins 10,20,30,40 reserved for Ground
	-- Pins 37,38,39 reserved for JTAG
	-- Pins 35-36 reserved for ComBlock Monitoring and Control.
	-- pullup pin B13 for interface with DACs.
	LEFT_CONNECTOR: in std_logic_vector(24 downto 1);
	
	--// Right connector: 40 pin male 2mm connector. 
	-- Pin 1 is reserved to forward clock to following module.
	-- Pins 10,20,30,40 reserved for Ground
	-- Pins 37,38,39 reserved for JTAG
	RIGHT_CONNECTOR: inout std_logic_vector(25 downto 1);


	--// SDRAM interface:
	-- For 256MB SDRAM or 1GB SDRAM assembly.
	SDRAM_A: out std_logic_vector(13 downto 0);
		-- Address lines. 
	SDRAM_DQ: inout std_logic_vector(63 downto 0);
		-- Bi-directional data lines
	SDRAM_CLK: out std_logic;
		-- not CLK
	BA0: out std_logic;
	BA1: out std_logic;
	CASN: out std_logic;
	RASN: out std_logic;
	WEN: out std_logic;
	C0N: out std_logic;
	C1N: out std_logic;
		-----------------------------------------------------
		--|                      |    SDRAM 0      SDRAM 1  |
		--|   A13    C0N    C1N  |   S0N   S1N    S0N   S1N |
		--|-------------------------------------------------|
		--|   0      0      0    |   0     0      1     1   |
		--|   0      0      1    |   0     1      1     1   |
		--|   0      1      0    |   1     0      1     1   |
		--|   0      1      1    |   1     1      1     1   |
		--|   1      0      0    |   1     1      0     0   |
		--|   1      0      1    |   1     1      0     1   |
		--|   1      1      0    |   1     1      1     0   |
		--|   1      1      1    |   1     1      1     1   |
		-----------------------------------------------------	


	--// Monitoring & Control interface Atmel controller
	UC_AD: inout std_logic_vector(7 downto 0);
	UC_WRN_IN: in std_logic;
	UC_RDN: in std_logic;
	UC_ALE_IN: in std_logic;
	UC_FLASH_CSN_IN: in std_logic;
		-- '0' when uC talks to the flash memory, '1' when otherwise (i.e. possibly
		-- talk to the FPGAs).


	--// Monitoring & Control: Serial ports interface
	RX_L: in std_logic;
	RX_R: in std_logic;
	TX_L: out std_logic;
	TX_R: out std_logic


	);
end entity;

architecture behavior of COM8000 is
--------------------------------------------------------------------------
-- COMPONENTS
--------------------------------------------------------------------------
component COMSCOPE is 
	port ( 
	--GLOBAL CLOCKS
   CLK : in std_logic;				-- Master clock for this FPGA
	ASYNC_RESET: in std_logic;		-- Asynchronous reset active high


	-- Choose between signed or unsigned representation
	SIGNED_REPRESENTATION_U_SIGNED_N: in std_logic;
		-- '0'  	Unsigned
		-- '1'  	Signed


	-- Input registers
	REG237: in std_logic_vector(7 downto 0);
	REG238: in std_logic_vector(7 downto 0);
	REG239: in std_logic_vector(7 downto 0);
	REG240: in std_logic_vector(7 downto 0);
	REG241: in std_logic_vector(7 downto 0);
	REG242: in std_logic_vector(7 downto 0);
	REG243: in std_logic_vector(7 downto 0);
	REG244: in std_logic_vector(7 downto 0);
	REG245: in std_logic_vector(7 downto 0);
	REG246: in std_logic_vector(7 downto 0);
	REG247: in std_logic_vector(7 downto 0);
	REG248: in std_logic_vector(7 downto 0);
	REG249: in std_logic_vector(7 downto 0);


	-- Trace 1 input signals
	-- 1-bit signals
	SIGNAL_1_BIT_1_1: in std_logic;
	SIGNAL_1_BIT_SAMPLE_CLK_1_1: in std_logic;
	SIGNAL_1_BIT_1_2: in std_logic;
	SIGNAL_1_BIT_SAMPLE_CLK_1_2: in std_logic;
	SIGNAL_1_BIT_1_3: in std_logic;
	SIGNAL_1_BIT_SAMPLE_CLK_1_3: in std_logic;
	SIGNAL_1_BIT_1_4: in std_logic;
	SIGNAL_1_BIT_SAMPLE_CLK_1_4: in std_logic;

	-- 2-bit signals
	SIGNAL_2_BIT_1_1: in std_logic_vector(1 downto 0);
	SIGNAL_2_BIT_SAMPLE_CLK_1_1: in std_logic;
	SIGNAL_2_BIT_1_2: in std_logic_vector(1 downto 0);
	SIGNAL_2_BIT_SAMPLE_CLK_1_2: in std_logic;
	SIGNAL_2_BIT_1_3: in std_logic_vector(1 downto 0);
	SIGNAL_2_BIT_SAMPLE_CLK_1_3: in std_logic;
	SIGNAL_2_BIT_1_4: in std_logic_vector(1 downto 0);
	SIGNAL_2_BIT_SAMPLE_CLK_1_4: in std_logic;

	-- 4-bit signals
	SIGNAL_4_BIT_1_1: in std_logic_vector(3 downto 0);
	SIGNAL_4_BIT_SAMPLE_CLK_1_1: in std_logic;
	SIGNAL_4_BIT_1_2: in std_logic_vector(3 downto 0);
	SIGNAL_4_BIT_SAMPLE_CLK_1_2: in std_logic;
	SIGNAL_4_BIT_1_3: in std_logic_vector(3 downto 0);
	SIGNAL_4_BIT_SAMPLE_CLK_1_3: in std_logic;
	SIGNAL_4_BIT_1_4: in std_logic_vector(3 downto 0);
	SIGNAL_4_BIT_SAMPLE_CLK_1_4: in std_logic;

	-- 8-bit signals
	SIGNAL_8_BIT_1_1: in std_logic_vector(7 downto 0);
	SIGNAL_8_BIT_SAMPLE_CLK_1_1: in std_logic;
	SIGNAL_8_BIT_1_2: in std_logic_vector(7 downto 0);
	SIGNAL_8_BIT_SAMPLE_CLK_1_2: in std_logic;
	SIGNAL_8_BIT_1_3: in std_logic_vector(7 downto 0);
	SIGNAL_8_BIT_SAMPLE_CLK_1_3: in std_logic;
	SIGNAL_8_BIT_1_4: in std_logic_vector(7 downto 0);
	SIGNAL_8_BIT_SAMPLE_CLK_1_4: in std_logic;

	-- 16-bit signals
	SIGNAL_16_BIT_1_1: in std_logic_vector(15 downto 0);
	SIGNAL_16_BIT_SAMPLE_CLK_1_1: in std_logic;
	SIGNAL_16_BIT_1_2: in std_logic_vector(15 downto 0);
	SIGNAL_16_BIT_SAMPLE_CLK_1_2: in std_logic;
	SIGNAL_16_BIT_1_3: in std_logic_vector(15 downto 0);
	SIGNAL_16_BIT_SAMPLE_CLK_1_3: in std_logic;
	SIGNAL_16_BIT_1_4: in std_logic_vector(15 downto 0);
	SIGNAL_16_BIT_SAMPLE_CLK_1_4: in std_logic;
	-- Add signals as necessary (up to 127 signals total per trace)


	-- Trace 2 input signals
	-- 1-bit signals
	SIGNAL_1_BIT_2_1: in std_logic;
	SIGNAL_1_BIT_SAMPLE_CLK_2_1: in std_logic;
	SIGNAL_1_BIT_2_2: in std_logic;
	SIGNAL_1_BIT_SAMPLE_CLK_2_2: in std_logic;
	SIGNAL_1_BIT_2_3: in std_logic;
	SIGNAL_1_BIT_SAMPLE_CLK_2_3: in std_logic;
	SIGNAL_1_BIT_2_4: in std_logic;
	SIGNAL_1_BIT_SAMPLE_CLK_2_4: in std_logic;

	-- 2-bit signals
	SIGNAL_2_BIT_2_1: in std_logic_vector(1 downto 0);
	SIGNAL_2_BIT_SAMPLE_CLK_2_1: in std_logic;
	SIGNAL_2_BIT_2_2: in std_logic_vector(1 downto 0);
	SIGNAL_2_BIT_SAMPLE_CLK_2_2: in std_logic;
	SIGNAL_2_BIT_2_3: in std_logic_vector(1 downto 0);
	SIGNAL_2_BIT_SAMPLE_CLK_2_3: in std_logic;
	SIGNAL_2_BIT_2_4: in std_logic_vector(1 downto 0);
	SIGNAL_2_BIT_SAMPLE_CLK_2_4: in std_logic;

	-- 4-bit signals
	SIGNAL_4_BIT_2_1: in std_logic_vector(3 downto 0);
	SIGNAL_4_BIT_SAMPLE_CLK_2_1: in std_logic;
	SIGNAL_4_BIT_2_2: in std_logic_vector(3 downto 0);
	SIGNAL_4_BIT_SAMPLE_CLK_2_2: in std_logic;
	SIGNAL_4_BIT_2_3: in std_logic_vector(3 downto 0);
	SIGNAL_4_BIT_SAMPLE_CLK_2_3: in std_logic;
	SIGNAL_4_BIT_2_4: in std_logic_vector(3 downto 0);
	SIGNAL_4_BIT_SAMPLE_CLK_2_4: in std_logic;

	-- 8-bit signals
	SIGNAL_8_BIT_2_1: in std_logic_vector(7 downto 0);
	SIGNAL_8_BIT_SAMPLE_CLK_2_1: in std_logic;
	SIGNAL_8_BIT_2_2: in std_logic_vector(7 downto 0);
	SIGNAL_8_BIT_SAMPLE_CLK_2_2: in std_logic;
	SIGNAL_8_BIT_2_3: in std_logic_vector(7 downto 0);
	SIGNAL_8_BIT_SAMPLE_CLK_2_3: in std_logic;
	SIGNAL_8_BIT_2_4: in std_logic_vector(7 downto 0);
	SIGNAL_8_BIT_SAMPLE_CLK_2_4: in std_logic;

	-- 16-bit signals
	SIGNAL_16_BIT_2_1: in std_logic_vector(15 downto 0);
	SIGNAL_16_BIT_SAMPLE_CLK_2_1: in std_logic;
	SIGNAL_16_BIT_2_2: in std_logic_vector(15 downto 0);
	SIGNAL_16_BIT_SAMPLE_CLK_2_2: in std_logic;
	SIGNAL_16_BIT_2_3: in std_logic_vector(15 downto 0);
	SIGNAL_16_BIT_SAMPLE_CLK_2_3: in std_logic;
	SIGNAL_16_BIT_2_4: in std_logic_vector(15 downto 0);
	SIGNAL_16_BIT_SAMPLE_CLK_2_4: in std_logic;
	-- Add signals as necessary (up to 127 signals total per trace)


	-- Trace 3 input signals
	-- 1-bit signals
	SIGNAL_1_BIT_3_1: in std_logic;
	SIGNAL_1_BIT_SAMPLE_CLK_3_1: in std_logic;
	SIGNAL_1_BIT_3_2: in std_logic;
	SIGNAL_1_BIT_SAMPLE_CLK_3_2: in std_logic;
	SIGNAL_1_BIT_3_3: in std_logic;
	SIGNAL_1_BIT_SAMPLE_CLK_3_3: in std_logic;
	SIGNAL_1_BIT_3_4: in std_logic;
	SIGNAL_1_BIT_SAMPLE_CLK_3_4: in std_logic;

	-- 2-bit signals
	SIGNAL_2_BIT_3_1: in std_logic_vector(1 downto 0);
	SIGNAL_2_BIT_SAMPLE_CLK_3_1: in std_logic;
	SIGNAL_2_BIT_3_2: in std_logic_vector(1 downto 0);
	SIGNAL_2_BIT_SAMPLE_CLK_3_2: in std_logic;
	SIGNAL_2_BIT_3_3: in std_logic_vector(1 downto 0);
	SIGNAL_2_BIT_SAMPLE_CLK_3_3: in std_logic;
	SIGNAL_2_BIT_3_4: in std_logic_vector(1 downto 0);
	SIGNAL_2_BIT_SAMPLE_CLK_3_4: in std_logic;

	-- 4-bit signals
	SIGNAL_4_BIT_3_1: in std_logic_vector(3 downto 0);
	SIGNAL_4_BIT_SAMPLE_CLK_3_1: in std_logic;
	SIGNAL_4_BIT_3_2: in std_logic_vector(3 downto 0);
	SIGNAL_4_BIT_SAMPLE_CLK_3_2: in std_logic;
	SIGNAL_4_BIT_3_3: in std_logic_vector(3 downto 0);
	SIGNAL_4_BIT_SAMPLE_CLK_3_3: in std_logic;
	SIGNAL_4_BIT_3_4: in std_logic_vector(3 downto 0);
	SIGNAL_4_BIT_SAMPLE_CLK_3_4: in std_logic;

	-- 8-bit signals
	SIGNAL_8_BIT_3_1: in std_logic_vector(7 downto 0);
	SIGNAL_8_BIT_SAMPLE_CLK_3_1: in std_logic;
	SIGNAL_8_BIT_3_2: in std_logic_vector(7 downto 0);
	SIGNAL_8_BIT_SAMPLE_CLK_3_2: in std_logic;
	SIGNAL_8_BIT_3_3: in std_logic_vector(7 downto 0);
	SIGNAL_8_BIT_SAMPLE_CLK_3_3: in std_logic;
	SIGNAL_8_BIT_3_4: in std_logic_vector(7 downto 0);
	SIGNAL_8_BIT_SAMPLE_CLK_3_4: in std_logic;

	-- 16-bit signals
	SIGNAL_16_BIT_3_1: in std_logic_vector(15 downto 0);
	SIGNAL_16_BIT_SAMPLE_CLK_3_1: in std_logic;
	SIGNAL_16_BIT_3_2: in std_logic_vector(15 downto 0);
	SIGNAL_16_BIT_SAMPLE_CLK_3_2: in std_logic;
	SIGNAL_16_BIT_3_3: in std_logic_vector(15 downto 0);
	SIGNAL_16_BIT_SAMPLE_CLK_3_3: in std_logic;
	SIGNAL_16_BIT_3_4: in std_logic_vector(15 downto 0);
	SIGNAL_16_BIT_SAMPLE_CLK_3_4: in std_logic;
	-- Add signals as necessary (up to 127 signals total per trace)


	-- Trace 4 input signals
	-- 1-bit signals
	SIGNAL_1_BIT_4_1: in std_logic;
	SIGNAL_1_BIT_SAMPLE_CLK_4_1: in std_logic;
	SIGNAL_1_BIT_4_2: in std_logic;
	SIGNAL_1_BIT_SAMPLE_CLK_4_2: in std_logic;
	SIGNAL_1_BIT_4_3: in std_logic;
	SIGNAL_1_BIT_SAMPLE_CLK_4_3: in std_logic;
	SIGNAL_1_BIT_4_4: in std_logic;
	SIGNAL_1_BIT_SAMPLE_CLK_4_4: in std_logic;

	-- 2-bit signals
	SIGNAL_2_BIT_4_1: in std_logic_vector(1 downto 0);
	SIGNAL_2_BIT_SAMPLE_CLK_4_1: in std_logic;
	SIGNAL_2_BIT_4_2: in std_logic_vector(1 downto 0);
	SIGNAL_2_BIT_SAMPLE_CLK_4_2: in std_logic;
	SIGNAL_2_BIT_4_3: in std_logic_vector(1 downto 0);
	SIGNAL_2_BIT_SAMPLE_CLK_4_3: in std_logic;
	SIGNAL_2_BIT_4_4: in std_logic_vector(1 downto 0);
	SIGNAL_2_BIT_SAMPLE_CLK_4_4: in std_logic;

	-- 4-bit signals
	SIGNAL_4_BIT_4_1: in std_logic_vector(3 downto 0);
	SIGNAL_4_BIT_SAMPLE_CLK_4_1: in std_logic;
	SIGNAL_4_BIT_4_2: in std_logic_vector(3 downto 0);
	SIGNAL_4_BIT_SAMPLE_CLK_4_2: in std_logic;
	SIGNAL_4_BIT_4_3: in std_logic_vector(3 downto 0);
	SIGNAL_4_BIT_SAMPLE_CLK_4_3: in std_logic;
	SIGNAL_4_BIT_4_4: in std_logic_vector(3 downto 0);
	SIGNAL_4_BIT_SAMPLE_CLK_4_4: in std_logic;

	-- 8-bit signals
	SIGNAL_8_BIT_4_1: in std_logic_vector(7 downto 0);
	SIGNAL_8_BIT_SAMPLE_CLK_4_1: in std_logic;
	SIGNAL_8_BIT_4_2: in std_logic_vector(7 downto 0);
	SIGNAL_8_BIT_SAMPLE_CLK_4_2: in std_logic;
	SIGNAL_8_BIT_4_3: in std_logic_vector(7 downto 0);
	SIGNAL_8_BIT_SAMPLE_CLK_4_3: in std_logic;
	SIGNAL_8_BIT_4_4: in std_logic_vector(7 downto 0);
	SIGNAL_8_BIT_SAMPLE_CLK_4_4: in std_logic;

	-- 16-bit signals
	SIGNAL_16_BIT_4_1: in std_logic_vector(15 downto 0);
	SIGNAL_16_BIT_SAMPLE_CLK_4_1: in std_logic;
	SIGNAL_16_BIT_4_2: in std_logic_vector(15 downto 0);
	SIGNAL_16_BIT_SAMPLE_CLK_4_2: in std_logic;
	SIGNAL_16_BIT_4_3: in std_logic_vector(15 downto 0);
	SIGNAL_16_BIT_SAMPLE_CLK_4_3: in std_logic;
	SIGNAL_16_BIT_4_4: in std_logic_vector(15 downto 0);
	SIGNAL_16_BIT_SAMPLE_CLK_4_4: in std_logic;
	-- Add signals as necessary (up to 127 signals total per trace)



	-- Trigger signals
	-- 1-bit Triggers
	TRIGGER_1_BIT_1: in std_logic;
	TRIGGER_1_BIT_SAMPLE_CLK_1: in std_logic;
	TRIGGER_1_BIT_2: in std_logic;
	TRIGGER_1_BIT_SAMPLE_CLK_2: in std_logic;
	TRIGGER_1_BIT_3: in std_logic;
	TRIGGER_1_BIT_SAMPLE_CLK_3: in std_logic;
	TRIGGER_1_BIT_4: in std_logic;
	TRIGGER_1_BIT_SAMPLE_CLK_4: in std_logic;

	-- 2-bit Triggers
	TRIGGER_2_BIT_1: in std_logic_vector(1 downto 0);
	TRIGGER_2_BIT_SAMPLE_CLK_1: in std_logic;
	TRIGGER_2_BIT_2: in std_logic_vector(1 downto 0);
	TRIGGER_2_BIT_SAMPLE_CLK_2: in std_logic;
	TRIGGER_2_BIT_3: in std_logic_vector(1 downto 0);
	TRIGGER_2_BIT_SAMPLE_CLK_3: in std_logic;
	TRIGGER_2_BIT_4: in std_logic_vector(1 downto 0);
	TRIGGER_2_BIT_SAMPLE_CLK_4: in std_logic;

	-- 4-bit Triggers
	TRIGGER_4_BIT_1: in std_logic_vector(3 downto 0);
	TRIGGER_4_BIT_SAMPLE_CLK_1: in std_logic;
	TRIGGER_4_BIT_2: in std_logic_vector(3 downto 0);
	TRIGGER_4_BIT_SAMPLE_CLK_2: in std_logic;
	TRIGGER_4_BIT_3: in std_logic_vector(3 downto 0);
	TRIGGER_4_BIT_SAMPLE_CLK_3: in std_logic;
	TRIGGER_4_BIT_4: in std_logic_vector(3 downto 0);
	TRIGGER_4_BIT_SAMPLE_CLK_4: in std_logic;

	-- 8-bit Triggers
	TRIGGER_8_BIT_1: in std_logic_vector(7 downto 0);
	TRIGGER_8_BIT_SAMPLE_CLK_1: in std_logic;
	TRIGGER_8_BIT_2: in std_logic_vector(7 downto 0);
	TRIGGER_8_BIT_SAMPLE_CLK_2: in std_logic;
	TRIGGER_8_BIT_3: in std_logic_vector(7 downto 0);
	TRIGGER_8_BIT_SAMPLE_CLK_3: in std_logic;
	TRIGGER_8_BIT_4: in std_logic_vector(7 downto 0);
	TRIGGER_8_BIT_SAMPLE_CLK_4: in std_logic;

	-- 16-bit Triggers
	TRIGGER_16_BIT_1: in std_logic_vector(15 downto 0);
	TRIGGER_16_BIT_SAMPLE_CLK_1: in std_logic;
	TRIGGER_16_BIT_2: in std_logic_vector(15 downto 0);
	TRIGGER_16_BIT_SAMPLE_CLK_2: in std_logic;
	TRIGGER_16_BIT_3: in std_logic_vector(15 downto 0);
	TRIGGER_16_BIT_SAMPLE_CLK_3: in std_logic;
	TRIGGER_16_BIT_4: in std_logic_vector(15 downto 0);
	TRIGGER_16_BIT_SAMPLE_CLK_4: in std_logic;
	-- Add triggers as necessary (up to 127 trigger signals total)


	-- Output registers
	REG250: out std_logic_vector(7 downto 0);
	REG251: out std_logic_vector(7 downto 0);

	-- Other input signals
	TRIGGER_REARM_TOGGLE: in std_logic;
	FORCE_TRIGGER_TOGGLE: in std_logic;
	REG250_READ: in std_logic;
	START_CAPTURE_TOGGLE: in std_logic
);
end component;

--------------------------------------------------------------------------
-- SIGNALS
--------------------------------------------------------------------------
-- Suffix _D indicates a one CLK delayed version of the net with the same name
-- Suffix _E indicates an extended precision version of the net with the same name
-- Suffix _R indicates a reduced precision version of the net with the same name
-- Suffix _N indicates an inverted version of the net with the same name
-- Suffix _LOCAL indicates an exact version of the (output signal) net with the same name

--// CONSTANTS
signal ZERO: std_logic;
signal ONE: std_logic;
signal ZERO2: std_logic_vector(1 downto 0);
signal ZERO4: std_logic_vector(3 downto 0);
signal ZERO8: std_logic_vector(7 downto 0);
signal ZERO16: std_logic_vector(15 downto 0);

--// CLOCKS, RESETS
signal CLK_IN1B: std_logic;
signal CLK: std_logic;
signal ASYNC_RESET: std_logic;
signal RESET_COUNTER: std_logic_vector(8 downto 0) := "000000000";	-- initialize for simulations

--// INPUT SIGNALS
signal LEFT_CONNECTOR_D: std_logic_vector(24 downto 2);
signal DATA_I_IN: std_logic_vector(9 downto 0);
signal DATA_I_IN_E: std_logic_vector(11 downto 0);
signal DATA_Q_IN: std_logic_vector(9 downto 0);
signal DATA_Q_IN_E: std_logic_vector(11 downto 0);
signal SAMPLE_CLK_IN: std_logic;

--// ComScope traces
signal S1_1: std_logic_vector(7 downto 0);	
signal S1_1_CLK: std_logic;
signal S1_1B_CLK: std_logic;
signal S1_2: std_logic_vector(7 downto 0);	
signal S1_2_CLK: std_logic;
signal S1_2B_CLK: std_logic;
signal S1_3: std_logic_vector(7 downto 0);	
signal S1_3_CLK: std_logic;
signal S1_3B_CLK: std_logic;
signal S1_4: std_logic_vector(7 downto 0);	
signal S1_4_CLK: std_logic;
signal S1_4B_CLK: std_logic;

signal S2_1: std_logic_vector(7 downto 0);	
signal S2_1_CLK: std_logic;
signal S2_1B_CLK: std_logic;
signal S2_2: std_logic_vector(7 downto 0);	
signal S2_2_CLK: std_logic;
signal S2_2B_CLK: std_logic;
signal S2_3: std_logic_vector(7 downto 0);	
signal S2_3_CLK: std_logic;
signal S2_3B_CLK: std_logic;
signal S2_4: std_logic_vector(7 downto 0);	
signal S2_4_CLK: std_logic;
signal S2_4B_CLK: std_logic;

signal T1: std_logic;
signal T1_CLK: std_logic;

signal TRIGGER_REARM_TOGGLE: std_logic:= '0';
signal FORCE_TRIGGER_TOGGLE: std_logic:= '0';
signal START_CAPTURE_TOGGLE: std_logic:= '0';
signal REG250_READ: std_logic;


--// Atmel uc interface
signal UC_ALE: std_logic;
signal UC_WRN: std_logic;
signal UC_WRN_B: std_logic;
signal UC_FLASH_CSN: std_logic;
signal UC_ADDRESS: std_logic_vector(7 downto 0);
signal CONFIG_CHANGE_TOGGLE: std_logic;
signal CONFIG_CHANGE_TOGGLE_D: std_logic;
signal CONFIG_CHANGE_TOGGLE_D2: std_logic;
signal CONFIG_CHANGE_PULSE: std_logic;

--// Control Registers
signal REG0 : std_logic_vector(7 downto 0) := x"00";	
signal REG1 : std_logic_vector(7 downto 0) := x"00";	
signal REG2 : std_logic_vector(7 downto 0) := x"00";	
signal REG3 : std_logic_vector(7 downto 0) := x"00";	
signal REG4 : std_logic_vector(7 downto 0) := x"01";	
signal REG5 : std_logic_vector(7 downto 0) := x"00";	
signal REG6 : std_logic_vector(7 downto 0) := x"00"; 	
signal REG7 : std_logic_vector(7 downto 0) := x"00";	
signal REG8 : std_logic_vector(7 downto 0) := x"00";	
signal REG9 : std_logic_vector(7 downto 0) := x"00";	
signal REG10: std_logic_vector(7 downto 0) := x"00";	
signal REG11: std_logic_vector(7 downto 0) := x"00";	
signal REG12: std_logic_vector(7 downto 0) := x"00";	

--// Status Registers
signal SREG0: std_logic_vector(7 downto 0);					
signal SREG1: std_logic_vector(7 downto 0);					
signal SREG2: std_logic_vector(7 downto 0);					
signal SREG3: std_logic_vector(7 downto 0);					
signal SREG4: std_logic_vector(7 downto 0);					
signal SREG5: std_logic_vector(7 downto 0);					
signal SREG6: std_logic_vector(7 downto 0);					
signal SREG7: std_logic_vector(7 downto 0);					
signal SREG8: std_logic_vector(7 downto 0);					
signal SREG9: std_logic_vector(7 downto 0);					
signal REG255: std_logic_vector(7 downto 0);

--// ComScope control registers
signal REG237: std_logic_vector(7 downto 0):= "00000000";
signal REG238: std_logic_vector(7 downto 0):= "00000000";
signal REG239: std_logic_vector(7 downto 0):= "00000000";
signal REG240: std_logic_vector(7 downto 0):= "00001101";	-- Trace 1 signal selection
signal REG241: std_logic_vector(7 downto 0):= "00000000";	-- Trigger position, decimation
signal REG242: std_logic_vector(7 downto 0):= "00001110";	-- Trace 2 signal selection
signal REG243: std_logic_vector(7 downto 0):= "00000000";	-- Trigger position, decimation
signal REG244: std_logic_vector(7 downto 0):= "00000000";	-- Trace 3 signal selection
signal REG245: std_logic_vector(7 downto 0):= "00000000";	-- Trigger position, decimation
signal REG246: std_logic_vector(7 downto 0):= "00000000";	-- Trace 4 signal selection
signal REG247: std_logic_vector(7 downto 0):= "00000000";	-- Trigger position, decimation
signal REG248: std_logic_vector(7 downto 0):= "10001101";	-- Trigger edge, trigger signal
signal REG249: std_logic_vector(7 downto 0):= "10111111";	-- Trigger threshold
signal REG250: std_logic_vector(7 downto 0):= "00000000";
signal REG251: std_logic_vector(7 downto 0):= "00000000";
  
--// FPGA option and revision (ASCII format)
--// TODO: INSERT AN OPTION AND REVISION CODE BELOW. These codes can be 
-- read using the ComBlock Control Center (click on the settings button, third button from the left)
-- For example:
-- '0' = 48
-- 'A' = 65
constant OPTION: std_logic_vector(7 downto 0) := x"20";	-- '-'  
constant REVISION: std_logic_vector(7 downto 0) := x"31";	-- '1' 
--------------------------------------------------------------------------
-- IMPLEMENTATION
--------------------------------------------------------------------------
begin   
ZERO16 <= (others => '0');


--------------------------------------------------------------------------
-- RESET AND CLOCKS
--------------------------------------------------------------------------

--// RESET ---------------------
-- Generate a reset pulse after power up. 
-- The reset counter starts at 0 and counts to 256 where it stays.
-- The ASYNC_RESET is active while the counter stays below 256.
RESET_COUNTER_GEN_001: process(CLK)
begin
	if rising_edge(CLK) then
		if(RESET_COUNTER(8) = '0') then
			RESET_COUNTER <= RESET_COUNTER + 1;
	  	end if;
  	end if;
end process;

ASYNC_RESET <= not(RESET_COUNTER(8));


--// CLOCKS
BUFFER_001: IBUFG port map (I => CLK_IN1, O => CLK_IN1B);	-- internal clock
BUFFER_003: IBUFG port map(I => UC_FLASH_CSN_IN, O => UC_FLASH_CSN);  
BUFFER_004: IBUFG port map(I => UC_ALE_IN, O => UC_ALE);  
BUFFER_005: IBUF port map(I => UC_WRN_IN, O => UC_WRN_B);  

-- Global buffers
BUFG_001: BUFG port map(I => UC_WRN_B, O => UC_WRN);  
BUFG_002: BUFG port map(I => CLK_IN1B, O => CLK);  

---- Double internal clock CLK_IN1 from 40 MHz (clock source) to 80 MHz (synchronous CLK)
--CLK_DLL_001: CLKDLL port map(
--	RST => ZERO,
--	CLKIN => CLK_IN1B,
--	CLKFB => CLK2XG,
--	CLK0 => CLK1X,
--	CLK2X => CLK2X,
--	LOCKED => CLKDLL_LOCKED
--	);


--------------------------------------------------------------------------
-- CUSTOM CODE
-- INSERT YOUR CODE HERE
--------------------------------------------------------------------------




--------------------------------------------------------------------------
-- COMSCOPE TRACES
-- OPTIONAL SECTION. COULD BE DELETED.
--------------------------------------------------------------------------
-- define ComScope traces here as per the example below
--S1_1 <= DATA_I_IN(9 downto 2);
--S1_1_CLK <= SAMPLE_CLK_IN;
--S1_2 <= DATA2_I(9 downto 2);
--S1_2_CLK <= SAMPLE2_CLK;
--S1_3 <= DATA11_Q(9 downto 2);
--S1_3_CLK <= SAMPLE11_CLK;
--
--S2_1 <= DATA_Q_IN(9 downto 2);
--S2_1_CLK <= SAMPLE_CLK_IN;
--S2_2 <= DATA2_Q(9 downto 2);
--S2_2_CLK <= SAMPLE2_CLK;
--S2_3 <= DATA11_I(9 downto 2);
--S2_3_CLK <= SAMPLE11_CLK;

--// Comscope
S1_1B_CLK <= S1_1_CLK or REG240(7);	-- Nominal sampling clock or real-time processing clock
S1_2B_CLK <= S1_2_CLK or REG240(7);	-- Nominal sampling clock or real-time processing clock
S1_3B_CLK <= S1_3_CLK or REG240(7);	-- Nominal sampling clock or real-time processing clock
S1_4B_CLK <= S1_4_CLK or REG240(7);	-- Nominal sampling clock or real-time processing clock

S2_1B_CLK <= S2_1_CLK or REG242(7);	-- Nominal sampling clock or real-time processing clock
S2_2B_CLK <= S2_2_CLK or REG242(7);	-- Nominal sampling clock or real-time processing clock
S2_3B_CLK <= S2_3_CLK or REG242(7);	-- Nominal sampling clock or real-time processing clock
S2_4B_CLK <= S2_4_CLK or REG242(7);	-- Nominal sampling clock or real-time processing clock

COMSCOPE_001: COMSCOPE port map(
   CLK => CLK,
	ASYNC_RESET => ASYNC_RESET,
	SIGNED_REPRESENTATION_U_SIGNED_N => REG248(6),	-- trigger format signed(1) or unsigned(0)
	REG237 => REG237,
	REG238 => REG238,
	REG239 => REG239,
	REG240 => REG240,
	REG241 => REG241,
	REG242 => REG242,
	REG243 => REG243,
	REG244 => REG244,
	REG245 => REG245,
	REG246 => REG246,
	REG247 => REG247,
	REG248 => REG248,
	REG249 => REG249,
	SIGNAL_1_BIT_1_1 => ZERO,
	SIGNAL_1_BIT_SAMPLE_CLK_1_1 => ZERO,
	SIGNAL_1_BIT_1_2 => ZERO,
	SIGNAL_1_BIT_SAMPLE_CLK_1_2 => ZERO,
	SIGNAL_1_BIT_1_3 => ZERO,
	SIGNAL_1_BIT_SAMPLE_CLK_1_3 => ZERO,
	SIGNAL_1_BIT_1_4 => ZERO,
	SIGNAL_1_BIT_SAMPLE_CLK_1_4 => ZERO,
	SIGNAL_2_BIT_1_1 => ZERO2,
	SIGNAL_2_BIT_SAMPLE_CLK_1_1 => ZERO,
	SIGNAL_2_BIT_1_2 => ZERO2,
	SIGNAL_2_BIT_SAMPLE_CLK_1_2 => ZERO,
	SIGNAL_2_BIT_1_3 => ZERO2,
	SIGNAL_2_BIT_SAMPLE_CLK_1_3 => ZERO,
	SIGNAL_2_BIT_1_4 => ZERO2,
	SIGNAL_2_BIT_SAMPLE_CLK_1_4 => ZERO,
	SIGNAL_4_BIT_1_1 => ZERO4,
	SIGNAL_4_BIT_SAMPLE_CLK_1_1 => ZERO,
	SIGNAL_4_BIT_1_2 => ZERO4,
	SIGNAL_4_BIT_SAMPLE_CLK_1_2 => ZERO,
	SIGNAL_4_BIT_1_3 => ZERO4,
	SIGNAL_4_BIT_SAMPLE_CLK_1_3 => ZERO,
	SIGNAL_4_BIT_1_4 => ZERO4,
	SIGNAL_4_BIT_SAMPLE_CLK_1_4 => ZERO,
	SIGNAL_8_BIT_1_1 => S1_1,
	SIGNAL_8_BIT_SAMPLE_CLK_1_1 => S1_1B_CLK,
	SIGNAL_8_BIT_1_2 => S1_2,
	SIGNAL_8_BIT_SAMPLE_CLK_1_2 => S1_2B_CLK,
	SIGNAL_8_BIT_1_3 => S1_3,
	SIGNAL_8_BIT_SAMPLE_CLK_1_3 => S1_3B_CLK,
	SIGNAL_8_BIT_1_4 => S1_4,
	SIGNAL_8_BIT_SAMPLE_CLK_1_4 => S1_4B_CLK,
	SIGNAL_16_BIT_1_1 => ZERO16,
	SIGNAL_16_BIT_SAMPLE_CLK_1_1 => ZERO,
	SIGNAL_16_BIT_1_2 => ZERO16,
	SIGNAL_16_BIT_SAMPLE_CLK_1_2 => ZERO,
	SIGNAL_16_BIT_1_3 => ZERO16,
	SIGNAL_16_BIT_SAMPLE_CLK_1_3 => ZERO,
	SIGNAL_16_BIT_1_4 => ZERO16,
	SIGNAL_16_BIT_SAMPLE_CLK_1_4 => ZERO,
	SIGNAL_1_BIT_2_1 => ZERO,
	SIGNAL_1_BIT_SAMPLE_CLK_2_1 => ZERO,
	SIGNAL_1_BIT_2_2 => ZERO,
	SIGNAL_1_BIT_SAMPLE_CLK_2_2 => ZERO,
	SIGNAL_1_BIT_2_3 => ZERO,
	SIGNAL_1_BIT_SAMPLE_CLK_2_3 => ZERO,
	SIGNAL_1_BIT_2_4 => ZERO,
	SIGNAL_1_BIT_SAMPLE_CLK_2_4 => ZERO,
	SIGNAL_2_BIT_2_1 => ZERO2,
	SIGNAL_2_BIT_SAMPLE_CLK_2_1 => ZERO,
	SIGNAL_2_BIT_2_2 => ZERO2,
	SIGNAL_2_BIT_SAMPLE_CLK_2_2 => ZERO,
	SIGNAL_2_BIT_2_3 => ZERO2,
	SIGNAL_2_BIT_SAMPLE_CLK_2_3 => ZERO,
	SIGNAL_2_BIT_2_4 => ZERO2,
	SIGNAL_2_BIT_SAMPLE_CLK_2_4 => ZERO,
	SIGNAL_4_BIT_2_1 => ZERO4,
	SIGNAL_4_BIT_SAMPLE_CLK_2_1 => ZERO,
	SIGNAL_4_BIT_2_2 => ZERO4,
	SIGNAL_4_BIT_SAMPLE_CLK_2_2 => ZERO,
	SIGNAL_4_BIT_2_3 => ZERO4,
	SIGNAL_4_BIT_SAMPLE_CLK_2_3 => ZERO,
	SIGNAL_4_BIT_2_4 => ZERO4,
	SIGNAL_4_BIT_SAMPLE_CLK_2_4 => ZERO,
	SIGNAL_8_BIT_2_1 => S2_1,
	SIGNAL_8_BIT_SAMPLE_CLK_2_1 => S2_1B_CLK,
	SIGNAL_8_BIT_2_2 => S2_2,
	SIGNAL_8_BIT_SAMPLE_CLK_2_2 => S2_2B_CLK,
	SIGNAL_8_BIT_2_3 => S2_3,
	SIGNAL_8_BIT_SAMPLE_CLK_2_3 => S2_3B_CLK,
	SIGNAL_8_BIT_2_4 => S2_4,
	SIGNAL_8_BIT_SAMPLE_CLK_2_4 => S2_4B_CLK,
	SIGNAL_16_BIT_2_1 => ZERO16,
	SIGNAL_16_BIT_SAMPLE_CLK_2_1 => ZERO,
	SIGNAL_16_BIT_2_2 => ZERO16,
	SIGNAL_16_BIT_SAMPLE_CLK_2_2 => ZERO,
	SIGNAL_16_BIT_2_3 => ZERO16,
	SIGNAL_16_BIT_SAMPLE_CLK_2_3 => ZERO,
	SIGNAL_16_BIT_2_4 => ZERO16,
	SIGNAL_16_BIT_SAMPLE_CLK_2_4 => ZERO,
	SIGNAL_1_BIT_3_1 => ZERO,
	SIGNAL_1_BIT_SAMPLE_CLK_3_1 => ZERO,
	SIGNAL_1_BIT_3_2 => ZERO,
	SIGNAL_1_BIT_SAMPLE_CLK_3_2 => ZERO,
	SIGNAL_1_BIT_3_3 => ZERO,
	SIGNAL_1_BIT_SAMPLE_CLK_3_3 => ZERO,
	SIGNAL_1_BIT_3_4 => ZERO,
	SIGNAL_1_BIT_SAMPLE_CLK_3_4 => ZERO,
	SIGNAL_2_BIT_3_1 => ZERO2,
	SIGNAL_2_BIT_SAMPLE_CLK_3_1 => ZERO,
	SIGNAL_2_BIT_3_2 => ZERO2,
	SIGNAL_2_BIT_SAMPLE_CLK_3_2 => ZERO,
	SIGNAL_2_BIT_3_3 => ZERO2,
	SIGNAL_2_BIT_SAMPLE_CLK_3_3 => ZERO,
	SIGNAL_2_BIT_3_4 => ZERO2,
	SIGNAL_2_BIT_SAMPLE_CLK_3_4 => ZERO,
	SIGNAL_4_BIT_3_1 => ZERO4,
	SIGNAL_4_BIT_SAMPLE_CLK_3_1 => ZERO,
	SIGNAL_4_BIT_3_2 => ZERO4,
	SIGNAL_4_BIT_SAMPLE_CLK_3_2 => ZERO,
	SIGNAL_4_BIT_3_3 => ZERO4,
	SIGNAL_4_BIT_SAMPLE_CLK_3_3 => ZERO,
	SIGNAL_4_BIT_3_4 => ZERO4,
	SIGNAL_4_BIT_SAMPLE_CLK_3_4 => ZERO,
	SIGNAL_8_BIT_3_1 => ZERO8,
	SIGNAL_8_BIT_SAMPLE_CLK_3_1 => ZERO,
	SIGNAL_8_BIT_3_2 => ZERO8,
	SIGNAL_8_BIT_SAMPLE_CLK_3_2 => ZERO,
	SIGNAL_8_BIT_3_3 => ZERO8,
	SIGNAL_8_BIT_SAMPLE_CLK_3_3 => ZERO,
	SIGNAL_8_BIT_3_4 => ZERO8,
	SIGNAL_8_BIT_SAMPLE_CLK_3_4 => ZERO,
	SIGNAL_16_BIT_3_1 => ZERO16,
	SIGNAL_16_BIT_SAMPLE_CLK_3_1 => ZERO,
	SIGNAL_16_BIT_3_2 => ZERO16,
	SIGNAL_16_BIT_SAMPLE_CLK_3_2 => ZERO,
	SIGNAL_16_BIT_3_3 => ZERO16,
	SIGNAL_16_BIT_SAMPLE_CLK_3_3 => ZERO,
	SIGNAL_16_BIT_3_4 => ZERO16,
	SIGNAL_16_BIT_SAMPLE_CLK_3_4 => ZERO,
	SIGNAL_1_BIT_4_1 => ZERO,
	SIGNAL_1_BIT_SAMPLE_CLK_4_1 => ZERO,
	SIGNAL_1_BIT_4_2 => ZERO,
	SIGNAL_1_BIT_SAMPLE_CLK_4_2 => ZERO,
	SIGNAL_1_BIT_4_3 => ZERO,
	SIGNAL_1_BIT_SAMPLE_CLK_4_3 => ZERO,
	SIGNAL_1_BIT_4_4 => ZERO,
	SIGNAL_1_BIT_SAMPLE_CLK_4_4 => ZERO,
	SIGNAL_2_BIT_4_1 => ZERO2,
	SIGNAL_2_BIT_SAMPLE_CLK_4_1 => ZERO,
	SIGNAL_2_BIT_4_2 => ZERO2,
	SIGNAL_2_BIT_SAMPLE_CLK_4_2 => ZERO,
	SIGNAL_2_BIT_4_3 => ZERO2,
	SIGNAL_2_BIT_SAMPLE_CLK_4_3 => ZERO,
	SIGNAL_2_BIT_4_4 => ZERO2,
	SIGNAL_2_BIT_SAMPLE_CLK_4_4 => ZERO,
	SIGNAL_4_BIT_4_1 => ZERO4,
	SIGNAL_4_BIT_SAMPLE_CLK_4_1 => ZERO,
	SIGNAL_4_BIT_4_2 => ZERO4,
	SIGNAL_4_BIT_SAMPLE_CLK_4_2 => ZERO,
	SIGNAL_4_BIT_4_3 => ZERO4,
	SIGNAL_4_BIT_SAMPLE_CLK_4_3 => ZERO,
	SIGNAL_4_BIT_4_4 => ZERO4,
	SIGNAL_4_BIT_SAMPLE_CLK_4_4 => ZERO,
	SIGNAL_8_BIT_4_1 => ZERO8,
	SIGNAL_8_BIT_SAMPLE_CLK_4_1 => ZERO,
	SIGNAL_8_BIT_4_2 => ZERO8,
	SIGNAL_8_BIT_SAMPLE_CLK_4_2 => ZERO,
	SIGNAL_8_BIT_4_3 => ZERO8,
	SIGNAL_8_BIT_SAMPLE_CLK_4_3 => ZERO,
	SIGNAL_8_BIT_4_4 => ZERO8,
	SIGNAL_8_BIT_SAMPLE_CLK_4_4 => ZERO,
	SIGNAL_16_BIT_4_1 => ZERO16,
	SIGNAL_16_BIT_SAMPLE_CLK_4_1 => ZERO,
	SIGNAL_16_BIT_4_2 => ZERO16,
	SIGNAL_16_BIT_SAMPLE_CLK_4_2 => ZERO,
	SIGNAL_16_BIT_4_3 => ZERO16,
	SIGNAL_16_BIT_SAMPLE_CLK_4_3 => ZERO,
	SIGNAL_16_BIT_4_4 => ZERO16,
	SIGNAL_16_BIT_SAMPLE_CLK_4_4 => ZERO,
	TRIGGER_1_BIT_1 => T1,
	TRIGGER_1_BIT_SAMPLE_CLK_1 => T1_CLK,
	TRIGGER_1_BIT_2 => ZERO,
	TRIGGER_1_BIT_SAMPLE_CLK_2 => ZERO,
	TRIGGER_1_BIT_3 => ZERO,
	TRIGGER_1_BIT_SAMPLE_CLK_3 => ZERO,
	TRIGGER_1_BIT_4 => ZERO,
	TRIGGER_1_BIT_SAMPLE_CLK_4 => ZERO,
	TRIGGER_2_BIT_1 => ZERO2,
	TRIGGER_2_BIT_SAMPLE_CLK_1 => ZERO,
	TRIGGER_2_BIT_2 => ZERO2,
	TRIGGER_2_BIT_SAMPLE_CLK_2 => ZERO,
	TRIGGER_2_BIT_3 => ZERO2,
	TRIGGER_2_BIT_SAMPLE_CLK_3 => ZERO,
	TRIGGER_2_BIT_4 => ZERO2,
	TRIGGER_2_BIT_SAMPLE_CLK_4 => ZERO,
	TRIGGER_4_BIT_1 => ZERO4,
	TRIGGER_4_BIT_SAMPLE_CLK_1 => ZERO,
	TRIGGER_4_BIT_2 => ZERO4,
	TRIGGER_4_BIT_SAMPLE_CLK_2 => ZERO,
	TRIGGER_4_BIT_3 => ZERO4,
	TRIGGER_4_BIT_SAMPLE_CLK_3 => ZERO,
	TRIGGER_4_BIT_4 => ZERO4,
	TRIGGER_4_BIT_SAMPLE_CLK_4 => ZERO,
	TRIGGER_8_BIT_1 => ZERO8,
	TRIGGER_8_BIT_SAMPLE_CLK_1 => ZERO,
	TRIGGER_8_BIT_2 => ZERO8,
	TRIGGER_8_BIT_SAMPLE_CLK_2 => ZERO,
	TRIGGER_8_BIT_3 => ZERO8,
	TRIGGER_8_BIT_SAMPLE_CLK_3 => ZERO,
	TRIGGER_8_BIT_4 => ZERO8,
	TRIGGER_8_BIT_SAMPLE_CLK_4 => ZERO,
	TRIGGER_16_BIT_1 => ZERO16,
	TRIGGER_16_BIT_SAMPLE_CLK_1 => ZERO,
	TRIGGER_16_BIT_2 => ZERO16,
	TRIGGER_16_BIT_SAMPLE_CLK_2 => ZERO,
	TRIGGER_16_BIT_3 => ZERO16,
	TRIGGER_16_BIT_SAMPLE_CLK_3 => ZERO,
	TRIGGER_16_BIT_4 => ZERO16,
	TRIGGER_16_BIT_SAMPLE_CLK_4 => ZERO,
	REG250 => REG250,
	REG251 => REG251,
	TRIGGER_REARM_TOGGLE => TRIGGER_REARM_TOGGLE,
	FORCE_TRIGGER_TOGGLE => FORCE_TRIGGER_TOGGLE,
	REG250_READ => REG250_READ,
	START_CAPTURE_TOGGLE => START_CAPTURE_TOGGLE
);


--------------------------------------------------------------------------
--// MONITORING & CONTROL REGISTERS
--// Atmel microcontroller interface
-- Used for monitoring and control.
-- DO NOT MODIFY THIS SECTION
--------------------------------------------------------------------------

--// Serial interfaces are used for module-to-module communication.
-- Do not change this section unless indicated in the comments.
-- REG255 reserved for reading 4 serial port inputs
-- These signals are pulled high (to mimic stop bit).
REG255(0) <= RX_L	;	-- port 1 left
REG255(1) <= '1'	;	-- port 2 bottom (does not exist on this board)
REG255(2) <= RX_R	;	-- port 3 right
REG255(3) <= '1'; 	-- USB port, byte ready #  (does not exist on this board)
REG255(7 downto 4) <= "0000";


-- Address latch
UC_ADDR_001: process(ASYNC_RESET, UC_ALE, UC_FLASH_CSN, UC_AD)
begin
	if(ASYNC_RESET = '1') then
		UC_ADDRESS <= (others => '0');
	elsif falling_edge(UC_ALE) then
		if(UC_FLASH_CSN = '1') then
			UC_ADDRESS <= UC_AD;
	  	end if;
  	end if;
end process;


-- Read
UC_READ_001: process(UC_FLASH_CSN, UC_RDN, UC_ADDRESS, SREG0, SREG1, SREG2, 
	SREG3, SREG4, SREG5, REG255)
begin
	if(UC_FLASH_CSN = '1') and (UC_RDN = '0') then
		if(UC_ADDRESS(7 downto 2) = "111111") then
			if(UC_ADDRESS(1 downto 0) = "11") then
				UC_AD <= REG255;
			elsif(UC_ADDRESS(1 downto 0) = "01") then
				UC_AD <= REVISION;	-- reg 253 = fpga revision
			elsif(UC_ADDRESS(1 downto 0) = "00") then
				UC_AD <= OPTION;		-- reg 252 = fpga option
			else
				UC_AD <= "ZZZZZZZZ";
			end if;
		elsif(UC_ADDRESS = 0) then
			UC_AD <= SREG0;
		elsif(UC_ADDRESS = 1) then
			UC_AD <= SREG1;
		elsif(UC_ADDRESS = 2) then
			UC_AD <= SREG2;
		elsif(UC_ADDRESS = 3) then
			UC_AD <= SREG3;
		elsif(UC_ADDRESS = 4) then
			UC_AD <= SREG4;
		elsif(UC_ADDRESS = 5) then
			UC_AD <= SREG5;
		elsif(UC_ADDRESS = 6) then
			UC_AD <= SREG6;
		elsif(UC_ADDRESS = 7) then
			UC_AD <= SREG7;
--		elsif(UC_ADDRESS = 8) then
--			UC_AD <= SREG8;
--		elsif(UC_ADDRESS = 9) then
--			UC_AD <= SREG9;
		elsif (UC_ADDRESS = 251) then
			UC_AD <= REG251;		-- reg 251 =comscope  status
		elsif (UC_ADDRESS = 250) then
			UC_AD <= REG250;		-- reg 250 = comscope data byte
		else
			UC_AD <= "ZZZZZZZZ";
		end if;
	else
		UC_AD <= "ZZZZZZZZ";
 	end if;
end process;

-- detect when address 250 is being read
REG250_READ <= '1' when (UC_FLASH_CSN = '1') and (UC_RDN = '0') and (UC_ADDRESS = 250) else '0';

-- Write
UC_WRITE_001: process(ASYNC_RESET, UC_FLASH_CSN, UC_WRN, UC_ADDRESS, UC_AD)
begin
	if(ASYNC_RESET = '1') then
		-- CAREFUL!!!!!! 
		-- when the external clock comes from a bigger FPGA, the ASYNC_RESET
		-- may arrive here AFTER the Atmel uC has configured the registers.
		-- We should avoid resetting the registers.
		TX_L <= '1';	-- stop bit
		TX_R <= '1';
	 elsif falling_edge(UC_WRN) then
		if(UC_FLASH_CSN = '1') then
			if (UC_ADDRESS(7 downto 5) = "000") then
				if (UC_ADDRESS(4 downto 0) = "00000") then
					REG0 <= UC_AD;
				end if;
				if (UC_ADDRESS(4 downto 0) = "00001") then
					REG1 <= UC_AD;
				end if;
				if (UC_ADDRESS(4 downto 0) = "00010") then
					REG2 <= UC_AD;
				end if;
				if (UC_ADDRESS(4 downto 0) = "00011") then
					REG3 <= UC_AD;
				end if;
				if (UC_ADDRESS(4 downto 0) = "00100") then
					REG4 <= UC_AD;
				end if;
				if (UC_ADDRESS(4 downto 0) = "00101") then
					REG5 <= UC_AD;
				end if;
				if (UC_ADDRESS(4 downto 0) = "00110") then
					REG6 <= UC_AD;
				end if;
				if (UC_ADDRESS(4 downto 0) = "00111") then
					REG7 <= UC_AD;
				end if;
				if (UC_ADDRESS(4 downto 0) = "01000") then
					REG8 <= UC_AD;
				end if;
				if (UC_ADDRESS(4 downto 0) = "01001") then
					REG9 <= UC_AD;
				end if;
				if (UC_ADDRESS(4 downto 0) = "01010") then
					REG10 <= UC_AD;
				end if;
				if (UC_ADDRESS(4 downto 0) = "01011") then
					REG11 <= UC_AD;
				end if;
				if (UC_ADDRESS(4 downto 0) = "01100") then
					REG12 <= UC_AD;
					CONFIG_CHANGE_TOGGLE <= not CONFIG_CHANGE_TOGGLE;	
					-- Configuration change toggles upon writing the last control register
				end if;
			end if;
			if (UC_ADDRESS = "11111110") then
				-- REG254 reserved for output serial
				TX_L <= UC_AD(0);
				TX_R <= UC_AD(2);
			end if;
		end if;
	end if;
end process;

-- Reclock CONFIG_CHANGE_TOGGLE to be synchronous with processing clock CLK
RECLOCK_005: process(CLK, CONFIG_CHANGE_TOGGLE)
begin
	if rising_edge(CLK) then
		CONFIG_CHANGE_TOGGLE_D <= CONFIG_CHANGE_TOGGLE;
		CONFIG_CHANGE_TOGGLE_D2 <= CONFIG_CHANGE_TOGGLE_D;
		if(CONFIG_CHANGE_TOGGLE_D /= CONFIG_CHANGE_TOGGLE_D2) then
			CONFIG_CHANGE_PULSE <= '1';
		else
			CONFIG_CHANGE_PULSE <= '0';
		end if;
	end if;
end process;


--// reserved COMSCOPE
UC_WRITE_002: process(ASYNC_RESET, UC_FLASH_CSN, UC_WRN, UC_ADDRESS, UC_AD, 
	FORCE_TRIGGER_TOGGLE, TRIGGER_REARM_TOGGLE, START_CAPTURE_TOGGLE)
begin
	if(ASYNC_RESET = '1') then
		-- Do not enter default values here. Any default value entered here will be
		-- overwritten by the microprocessor, typically within 10ms after the FPGA is configured.
	elsif falling_edge(UC_WRN) then
		if(UC_FLASH_CSN = '1') then
			-- Reserved for ComScope			
			if (UC_ADDRESS = 237) then
				REG237 <= UC_AD;	
			end if;
			if (UC_ADDRESS = 238) then
				REG238 <= UC_AD;	
			end if;
			if (UC_ADDRESS = 239) then
				REG239 <= UC_AD;
			end if;
			if (UC_ADDRESS = 240) then
				REG240 <= UC_AD;	
			end if;
			if (UC_ADDRESS = 241) then
				REG241 <= UC_AD;	
			end if;
			if (UC_ADDRESS = 242) then
				REG242 <= UC_AD;	
			end if;
			if (UC_ADDRESS = 243) then
				REG243 <= UC_AD;	
			end if;
			if (UC_ADDRESS = 244) then
				REG244 <= UC_AD;	
			end if;
			if (UC_ADDRESS = 245) then
				REG245 <= UC_AD;	
			end if;
			if (UC_ADDRESS = 246) then
				REG246 <= UC_AD;	
			end if;
			if (UC_ADDRESS = 247) then
				REG247 <= UC_AD;	
			end if;
			if (UC_ADDRESS = 248) then
				REG248 <= UC_AD;	
			end if;
			if (UC_ADDRESS = 249) then
				REG249 <= UC_AD;	
			end if;
			if (UC_ADDRESS = 250) then 
				-- detect one-shot force trigger	
				if(UC_AD(0) = '1') then	
					FORCE_TRIGGER_TOGGLE <= not(FORCE_TRIGGER_TOGGLE);
				end if;	
				-- detect one-shot re-arm	
				if(UC_AD(1) = '1') then	
					TRIGGER_REARM_TOGGLE <= not(TRIGGER_REARM_TOGGLE);
				end if;	
				-- start capturing again
				if(UC_AD(2) = '1') then	
					START_CAPTURE_TOGGLE <= not (START_CAPTURE_TOGGLE);
				end if;	
			end if;
		end if;
	end if;
end process;

end behavior;